/*
 *				**** FILL IN THE BLANKS ****
 *
 * This program contains the routines for the fill in the blank questions
 *
 */

#include "quiz.h"

#ifdef __MSDOS__
#include <conio.h>

static int sound_on = 1;
#endif

#define MAXWORDS	1000

static	int xpos=0;

fill_blanks(FILE *fp)
{
	int num_quest=0, num_right=0, errs, percent;
	char buf[BUFSIZ];

	while (1)
	{
		printf("What percent of the words do you want blank? ");
		gets(buf);
		percent = atoi(buf);
		if (percent > 0 && percent <= 100) break;
		printf("Invalid response.  Please try again.\n");
	}

	do
	{
		errs = do_fill_blanks(fp, percent);
		num_quest++;
		if (errs)
		{
			printf("Sorry, you missed %d words\n", errs);
		} else
		{
			printf("Very good!\n");
			num_right++;
		}
		printf("You have %d out of %d so far.\n", num_right, num_quest);
		printf("\nPress ENTER to continue, or type 'q' to quit: ");
		gets(buf);
	} while (*buf != 'q' && *buf != 'Q');
	return ( (num_right * 100) / num_quest);
}

do_fill_blanks(FILE *fp, int percent)
{
	char *Words[MAXWORDS];	/* words in quotation */
	char *Punct[MAXWORDS];	/* punctuation following word */
	int  Blank[MAXWORDS];	/* what words are to be blank */
	int	 i, j, k;
	int quest, num_words=0, errs=0, num_blanks;
	char buf[BUFSIZ], *p, *q, c;

	get_nums(1);
	quest = Nums[0];

	fseek(fp, Quiz[quest].q_d, 0);		/* point to beginning of question */
	if (! get_line(buf, sizeof(buf), fp)) return 0;
	p = buf;

	while(1)
	{ /* read entire question & save one word at a time */
		while (isspace(*p)) p++;		/* find start of word */
		if (*p == '\0')
		{ /* end of line - get next line */
			if (! get_line(buf, sizeof(buf), fp)) break;	/* end of file */
			for (p=buf; isspace(*p); p++);		/* find first word */
			if (*p == '\0') break;				/* end of question */
		}

		/* p points to start of word.  Find end of word */
		for (q=p; isalnum(*q) || *q == '\''; q++);	/* find end of word */
		if (q > p)
		{ /* word found - copy it */
			c = *q;							/* save end of word */
			*q = '\0';						/* terminate word */
			Words[num_words] = copy_string(p);
			*q = c;							/* restore end of word */
		} else Words[num_words] = NULL;		/* no word - just punctuation */

		for (p = q; ispunct(*q); q++);	/* punctuation */
		if (q > p)
		{ /* punctuation found - save it */
			c = *q;
			*q = '\0';
			Punct[num_words] = copy_string(p);
			*q = c;
		} else Punct[num_words] = NULL;
		num_words++;
		p = q;
	}

	/* words found - now get words to blank */

	/* first, generate list of words */
	for (i=0; i < num_words; i++) Blank[i] = i;

	/* next, compute number of blanks to use */
	num_blanks = (num_words * percent) / 100;
	if (num_blanks == 0) num_blanks = 1;

	/* now get list of 'num_blanks' words to be blanked */
	if (num_blanks < num_words)
	{ /* at least one non-blank word - get random list of 'num_blanks' */
		for (i=0; i < num_blanks; i++)
		{ /* we need num_blanks blank words */
			j = random(num_words);	/* pick a random number */
			k = Blank[i];			/* old value */
			Blank[i] = Blank[j];	/* move value */
			Blank[j] = k;			/* swap numbers */
		}
	}

	put_line(fp, stdout, Quiz[quest].q_w);

	xpos = 0;

	for (i=0; i < num_words; i++)
	{ /* print word or ask for word */
		for (j=0; j < num_blanks; j++) if (Blank[j] == i) break;
		if (j < num_blanks)
		{ /* word is to be blanked */
			errs += get_word(Words[i]);
#ifdef __MSDOS__
			xpos += strlen(Words[i]);
			if (xpos > right_margin)
			{ /* line overflow - start new line */
				printf("\n");
				xpos = strlen(Words[i]);
			}
			if (Punct[i] != NULL)
			{
				printf("%s", Punct[i]);
				xpos += strlen(Punct[i]);
			}
			printf(" ");
			xpos++;
#endif
		} else
		{ /* print out word */
			xpos += strlen(Words[i]);
			if (xpos > right_margin)
			{ /* line overflow - start new line */
				printf("\n");
				xpos = strlen(Words[i]);
			}
			if (Words[i] != NULL)
			{
				printf("%s", Words[i]);
			}
			if (Punct[i] != NULL)
			{
				printf("%s", Punct[i]);
				xpos += strlen(Punct[i]);
			}
			printf(" ");
			xpos++;
		}
	}
	printf("\n");

	/* free up memory */
	for (i=0; i < num_words; i++)
	{
		if (Words[i] != NULL)
		{
			free(Words[i]);
			Words[i] = NULL;
		}
		if (Punct[i] != NULL)
		{
			free(Punct[i]);
			Punct[i] = NULL;
		}
	}

	return errs;
}

get_word(char *word)
{
	int ret=0;

#ifdef __MSDOS__
/* PC version - get character at a time */
	int c;

	chirp();
	while (*word != '\0')
	{	/* get character at a time.  Only allow good characters */
		c = getch();
		if (c == '\t')
		{ /* toggle sound */
			sound_on = ! sound_on;
			continue;
		}
		if (c == '?' || c == *word || toupper(c) == *word || tolower(c) == *word)
		{ /* character matches - output it and go onto the next one */
			putch(*word++);
		} else wrong_sound();	/* wrong character */
	}
	chirp();
#else
	char buf[BUFSIZ];

	printf("\nEnter next word: ");
	gets(buf);
	if (stricmp(word, buf))
	{
		printf("Sorry - the word should be %s\n", word);
		ret++;
	}
	xpos = 0;
#endif

	return ret;
}

#ifdef __MSDOS__
/* special routines for MS-DOS */

bad_char()
{ /* user entered the wrong character */
	int i;

	for (i=1000; i > 100; i--) sound(i);
	nosound();
}

wrong_sound()
{
	int i;

	if (!sound_on) return;

	for (i=100; i > 50; i-=2)
	{
		sound(i);
		delay(20);
	}
	nosound();
}

chirp()
{
	int i;

	if (!sound_on) return;

	for (i=900; i < 1500; i+=10)
	{
		sound(i);
		delay(1);
	}
	nosound();
}

#endif
